@@ -1,7 +1,7 @@
 
 # Jython Database Specification API 2.0
 #
-# $Id: zxtest.py,v 1.5 2001/12/29 18:00:14 bzimmer Exp $
+# $Id: zxtest.py 2281 2003-04-10 20:17:43Z bzimmer $
 #
 # Copyright (c) 2001 brian zimmer <bzimmer@ziclix.com>
 
@@ -27,8 +27,8 @@ class zxCoreTestCase(runner.SQLTestCase):
 		connect = getattr(factory, self.factory.method)
 		return apply(connect, args, self.factory.keywords)
 
-	def cursor(self, dynamic=0):
-		c = self.db.cursor(dynamic)
+	def cursor(self, *args, **kws):
+		c = apply(self.db.cursor, args, kws)
 		if hasattr(self, "datahandler"):
 			c.datahandler = self.datahandler(c.datahandler)
 		return c
@@ -37,18 +37,18 @@ class zxJDBCTestCase(zxCoreTestCase):
 
 	def setUp(self):
 		zxCoreTestCase.setUp(self)
-		self.db = self.connect()
-		self.db.autocommit = 0
 
 		c = self.cursor()
 
 		try:
 			c.execute("drop table zxtesting")
+			self.db.commit()
 		except:
 			self.db.rollback()
 
 		try:
 			c.execute("create table zxtesting (id int not null, name varchar(32), state varchar(32), primary key (id))")
+			self.db.commit()
 			c.execute("insert into zxtesting (id, name, state) values (1, 'test0', 'il')")
 			c.execute("insert into zxtesting (id, name, state) values (2, 'test1', 'wi')")
 			c.execute("insert into zxtesting (id, name, state) values (3, 'test2', 'tx')")
@@ -101,15 +101,184 @@ class zxAPITestCase(zxJDBCTestCase):
 		finally:
 			c.close()
 
+	def testNoneQuery(self):
+		"""testing that executing None doesn't fail"""
+		c = self.cursor()
+		try:
+			c.execute(None)
+		finally:
+			c.close()
+
+	def _test_preparedstatement(self, dynamic):
+		c = self.cursor(dynamic)
+		try:
+			p = c.prepare("select * from zxtesting where id = ?")
+			for i in range(1, 8):
+				c.execute(p, (i,))
+				data = c.fetchall()
+				self.assertEquals(1, len(data))
+			assert not p.closed
+			p.close()
+			assert p.closed
+			self.assertRaises(zxJDBC.ProgrammingError, c.execute, p, (1,))
+		finally:
+			c.close()
+
+	def testStaticPrepare(self):
+		"""testing the prepare() functionality for static cursors"""
+		self._test_preparedstatement(0)
+
+	def testDynamicPrepare(self):
+		"""testing the prepare() functionality for dynamic cursors"""
+		self._test_preparedstatement(1)
+
+	def _test_cursorkeywords(self, *args, **kws):
+		c = self.cursor(*args, **kws)
+		try:
+			c.execute("select * from zxtesting")
+			data = c.fetchmany(1)
+			assert len(data) == 1, "expecting one row"
+		finally:
+			c.close()
+
+	def testCursorKeywords(self):
+		"""testing the creation of a cursor with keywords"""
+		self._test_cursorkeywords(dynamic=1)
+		self._test_cursorkeywords(dynamic=1,
+			rstype=zxJDBC.TYPE_SCROLL_INSENSITIVE,
+			rsconcur=zxJDBC.CONCUR_READ_ONLY
+		)
+		self._test_cursorkeywords(1,
+			rstype=zxJDBC.TYPE_SCROLL_INSENSITIVE,
+			rsconcur=zxJDBC.CONCUR_READ_ONLY
+		)
+		self._test_cursorkeywords(1,zxJDBC.TYPE_SCROLL_INSENSITIVE,zxJDBC.CONCUR_READ_ONLY)
+		self.assertRaises(TypeError, self.cursor, 1, zxJDBC.TYPE_SCROLL_INSENSITIVE)
+
+	def testFileLikeCursor(self):
+		"""testing the cursor as a file-like object"""
+		c = self.cursor()
+		try:
+			print >> c, "insert into zxtesting (id, name, state) values (100, 'test100', 'wa')"
+			print >> c, "insert into zxtesting (id, name, state) values (101, 'test101', 'co')"
+			print >> c, "insert into zxtesting (id, name, state) values (102, 'test102', 'or')"
+			self.db.commit()
+		finally:
+			c.close()
+
+		c = self.cursor()
+		try:
+			c.execute("select * from zxtesting where id in (100, 101, 102)")
+			f = c.fetchall()
+			self.assertEquals(3, len(f))
+		finally:
+			c.close()
+
+	def testIteration(self):
+		"""testing the iteration protocol"""
+		c = self.cursor()
+		try:
+			# first with a for loop
+			cnt = 0
+			c.execute("select * from zxtesting")
+			for a in c:
+				self.assertEquals(3, len(a))
+				cnt += 1
+			self.assertEquals(7, cnt)
+			# then with a while loop
+			cnt = 0
+			c.execute("select * from zxtesting")
+			while 1:
+				try:
+					self.assertEquals(3, len(c.next()))
+				except StopIteration:
+					break
+				cnt += 1
+			self.assertEquals(7, cnt)
+		finally:
+			c.close()
+
+	def testClosingCursor(self):
+		"""testing that a closed cursor throws an exception"""
+		c = self.cursor()
+		try:
+			c.execute("select * from zxtesting")
+		finally:
+			c.close()
+		self.assertRaises(zxJDBC.ProgrammingError, c.execute, ("select * from zxtesting",))
+
+	def testClosingConnectionWithOpenCursors(self):
+		"""testing that a closed connection closes any open cursors"""
+		c = self.cursor()
+		d = self.cursor()
+		e = self.cursor()
+		self.db.close()
+		# open a new connection so the tearDown can run
+		self.db = self.connect()
+		self.assertRaises(zxJDBC.ProgrammingError, c.execute, ("select * from zxtesting",))
+		self.assertRaises(zxJDBC.ProgrammingError, d.execute, ("select * from zxtesting",))
+		self.assertRaises(zxJDBC.ProgrammingError, e.execute, ("select * from zxtesting",))
+
+	def testNativeSQL(self):
+		"""testing the connection's ability to convert sql"""
+		sql = self.db.nativesql("select * from zxtesting where id = ?")
+		assert sql is not None
+		assert len(sql) > 0
+
+	def testTables(self):
+		"""testing cursor.tables()"""
+		c = self.cursor()
+		try:
+			c.tables(None, None, None, None)
+			# let's look for zxtesting
+			found = 0
+			while not found:
+				try:
+					found = "zxtesting" == c.next()[2].lower()
+				except StopIteration:
+					break
+			assert found, "expected to find 'zxtesting'"
+			c.tables(None, None, "zxtesting", None)
+			self.assertEquals(1, len(c.fetchall()))
+			c.tables(None, None, "zxtesting", ("TABLE",))
+			self.assertEquals(1, len(c.fetchall()))
+			c.tables(None, None, "zxtesting", ("table",))
+			self.assertEquals(1, len(c.fetchall()))
+		finally:
+			c.close()
+
 	def testColumns(self):
 		"""testing cursor.columns()"""
 		c = self.cursor()
 		try:
+			# deliberately copied so as to produce useful line numbers
+
 			c.columns(None, None, "zxtesting", None)
 			f = c.fetchall()
-			assert c.rowcount == 3, "columns() failed to report correct number of columns, expected [3], got [%d]" % (c.rowcount)
+			self.assertEquals(3, c.rowcount)
 			f.sort(lambda x, y: cmp(x[3], y[3]))
-			assert "name" == f[1][3].lower(), "expected [name], got [%s]" % (f[1][3].lower())
+			self.assertEquals("name", f[1][3].lower())
+
+			# if the db engine handles mixed case, then don't ask about a different
+			#  case because it will fail
+			if not self.db.__connection__.getMetaData().storesMixedCaseIdentifiers():
+				c.columns(None, None, "ZXTESTING", None)
+				f = c.fetchall()
+				self.assertEquals(3, c.rowcount)
+				f.sort(lambda x, y: cmp(x[3], y[3]))
+				self.assertEquals("name", f[1][3].lower())
+		finally:
+			c.close()
+
+	def testBestRow(self):
+		"""testing bestrow which finds the optimal set of columns that uniquely identify a row"""
+		c = self.cursor()
+		try:
+			# we're really just testing that this doesn't blow up
+			c.bestrow(None, None, "zxtesting")
+			f = c.fetchall()
+			if f: # we might as well see that it worked
+				self.assertEquals(1, len(f))
 		finally:
 			c.close()
 
@@ -127,6 +296,124 @@ class zxAPITestCase(zxJDBCTestCase):
 		finally:
 			c.close()
 
+	def _test_scrolling(self, dynamic=0):
+		if self.vendor.scroll:
+			c = self.cursor(dynamic,
+				rstype=getattr(zxJDBC, self.vendor.scroll),
+				rsconcur=zxJDBC.CONCUR_READ_ONLY
+			)
+		else:
+			c = self.cursor(dynamic)
+
+		try:
+			# set everything up
+			c.execute("select id, name, state from zxtesting order by id")
+			self.assertEquals(1, c.fetchone()[0])
+			self.assertEquals(2, c.fetchone()[0])
+			self.assertEquals(3, c.fetchone()[0])
+			# move back two and fetch the row again
+			c.scroll(-2)
+			self.assertEquals(2, c.fetchone()[0])
+			# move to the fifth row (0-based indexing)
+			c.scroll(4, "absolute")
+			self.assertEquals(5, c.fetchone()[0])
+			# move back to the start
+			c.scroll(-5)
+			self.assertEquals(1, c.fetchone()[0])
+			# move to the end
+			c.scroll(6, "absolute")
+			self.assertEquals(7, c.fetchone()[0])
+			# make sure we get an IndexError
+			self.assertRaises(IndexError, c.scroll, 1, "relative")
+			self.assertRaises(IndexError, c.scroll, -1, "absolute")
+			self.assertRaises(zxJDBC.ProgrammingError, c.scroll, 1, "somethingelsealltogether")
+		finally:
+			c.close()
+
+	def testDynamicCursorScrolling(self):
+		"""testing the ability to scroll a dynamic cursor"""
+		self._test_scrolling(1)
+
+	def testStaticCursorScrolling(self):
+		"""testing the ability to scroll a static cursor"""
+		self._test_scrolling(0)
+
+	def _test_rownumber(self, dynamic=0):
+		if self.vendor.scroll:
+			c = self.cursor(dynamic,
+				rstype=getattr(zxJDBC, self.vendor.scroll),
+				rsconcur=zxJDBC.CONCUR_READ_ONLY
+			)
+		else:
+			c = self.cursor(dynamic)
+
+		try:
+			if not dynamic:
+				# a dynamic cursor doesn't know if any rows really exist
+				# maybe the 'possibility' of rows should change .rownumber to 0?
+				c.execute("select * from zxtesting where 1=0")
+				self.assertEquals(0, c.rownumber)
+			c.execute("select * from zxtesting")
+			self.assertEquals(0, c.rownumber)
+			c.next()
+			self.assertEquals(1, c.rownumber)
+			c.next()
+			self.assertEquals(2, c.rownumber)
+			c.scroll(-1)
+			self.assertEquals(1, c.rownumber)
+			c.scroll(2, "absolute")
+			self.assertEquals(2, c.rownumber)
+			c.scroll(6, "absolute")
+			self.assertEquals(6, c.rownumber)
+		finally:
+			c.close()
+		self.assertEquals(None, c.rownumber)
+
+	def testStaticRownumber(self):
+		"""testing a static cursor's rownumber"""
+		self._test_rownumber(0)
+
+	def testDynamicRownumber(self):
+		"""testing a dynamic cursor's rownumber"""
+		self._test_rownumber(1)
+
+	def _test_rowcount(self, dynamic=0):
+		if self.vendor.scroll:
+			c = self.cursor(dynamic,
+				rstype=getattr(zxJDBC, self.vendor.scroll),
+				rsconcur=zxJDBC.CONCUR_READ_ONLY
+			)
+		else:
+			c = self.cursor(dynamic)
+
+		try:
+			c.execute("select * from zxtesting")
+			c.next()
+			c.next()
+			c.next()
+			if dynamic:
+				# dynamic cursors only know about the number of rows encountered
+				self.assertEquals(3, c.rowcount)
+			else:
+				self.assertEquals(7, c.rowcount)
+			c.scroll(-1)
+			# make sure they don't change just because we scrolled backwards
+			if dynamic:
+				# dynamic cursors only know about the number of rows encountered
+				self.assertEquals(3, c.rowcount)
+			else:
+				self.assertEquals(7, c.rowcount)
+		finally:
+			c.close()
+
+	def testStaticRowcount(self):
+		"""testing a static cursor's rowcount"""
+		self._test_rowcount(0)
+
+	def testDynamicRowcount(self):
+		"""testing a dynamic cursor's rowcount"""
+		self._test_rowcount(1)
+
 	def testTableTypeInfo(self):
 		"""testing cursor.gettabletypeinfo()"""
 		c = self.cursor()
@@ -205,7 +492,6 @@ class zxAPITestCase(zxJDBCTestCase):
 				assert len(f[0][0]) == len(data), "expected [%d], got [%d]" % (len(data), len(f[0][0]))
 				assert data == f[0][0], "failed to retrieve the same text as inserted"
 			except Exception, e:
-				print e
 				raise e
 		finally:
 			c.execute("drop table %s" % (self.table("texttable")[0]))
@@ -214,7 +500,7 @@ class zxAPITestCase(zxJDBCTestCase):
 			fp.close()
 			os.remove(fp.name)
 
-	def __calendar(self):
+	def calendar(self):
 		c = Calendar.getInstance()
 		c.setTime(JDate())
 		return c
@@ -224,7 +510,7 @@ class zxAPITestCase(zxJDBCTestCase):
 
 		# Java uses milliseconds and Python uses seconds, so adjust the time accordingly
 		# seeded with Java
-		c = self.__calendar()
+		c = self.calendar()
 		o = zxJDBC.DateFromTicks(c.getTime().getTime() / 1000L)
 		v = zxJDBC.Date(c.get(Calendar.YEAR), c.get(Calendar.MONTH) + 1, c.get(Calendar.DATE))
 		assert o.equals(v), "incorrect date conversion using java, got [%ld], expected [%ld]" % (v.getTime(), o.getTime())
@@ -242,7 +528,7 @@ class zxAPITestCase(zxJDBCTestCase):
 		# Java uses milliseconds and Python uses seconds, so adjust the time accordingly
 
 		# seeded with Java
-		c = self.__calendar()
+		c = self.calendar()
 		o = zxJDBC.TimeFromTicks(c.getTime().getTime() / 1000L)
 		v = zxJDBC.Time(c.get(Calendar.HOUR), c.get(Calendar.MINUTE), c.get(Calendar.SECOND))
 		assert o.equals(v), "incorrect date conversion using java, got [%ld], expected [%ld]" % (v.getTime(), o.getTime())
@@ -260,7 +546,7 @@ class zxAPITestCase(zxJDBCTestCase):
 		# Java uses milliseconds and Python uses seconds, so adjust the time accordingly
 
 		# seeded with Java
-		c = self.__calendar()
+		c = self.calendar()
 		o = zxJDBC.TimestampFromTicks(c.getTime().getTime() / 1000L)
 		v = zxJDBC.Timestamp(c.get(Calendar.YEAR), c.get(Calendar.MONTH) + 1, c.get(Calendar.DATE),
 			c.get(Calendar.HOUR), c.get(Calendar.MINUTE), c.get(Calendar.SECOND))
@@ -358,25 +644,28 @@ class zxAPITestCase(zxJDBCTestCase):
 		finally:
 			c.close()
 
-	def testUpdateCount(self, insert_only=0):
+	def testUpdateCount(self):
+		"""testing update count functionality"""
 		c = self.cursor()
 		try:
 			c.execute("insert into zxtesting values (?, ?, ?)", [(500, 'bz', 'or')])
 			assert c.updatecount == 1, "expected [1], got [%d]" % (c.updatecount)
 			c.execute("select * from zxtesting")
-			assert c.updatecount == -1, "expected updatecount to be -1 after query"
+			self.assertEquals(None, c.updatecount)
 			# there's a *feature* in the mysql engine where it returns 0 for delete if there is no
 			#  where clause, regardless of the actual value.  using a where clause forces it to calculate
 			#  the appropriate value
 			c.execute("delete from zxtesting where 1>0")
 			assert c.updatecount == 8, "expected [8], got [%d]" % (c.updatecount)
+			c.execute("update zxtesting set name = 'nothing'")
+			self.assertEquals(0, c.updatecount)
 		finally:
 			c.close()
 
-	def _test_time(self, (tabname, sql), factory, values, _type, _cmp=cmp):
+	def _test_time(self, (tabname, sql), factory, values, _type, _cmp=cmp, datahandler=None):
 		c = self.cursor()
+		if datahandler: c.datahandler = datahandler(c.datahandler)
 		try:
-
 			c.execute(sql)
 			dates = map(lambda x, f=factory: apply(f, x), values)
 			for a in dates:
@@ -393,8 +682,7 @@ class zxAPITestCase(zxJDBCTestCase):
 
 			c.execute("select * from %s" % (tabname))
 			f = c.fetchall()
-			assert len(f) == len(dates) - 1, "expected length [%d], got [%d]" % (len(dates) - 1, len(f))
-
+			self.assertEquals(len(f), len(dates) - 1)
 		finally:
 			c.execute("drop table %s" % (tabname))
 			c.close()
@@ -430,16 +718,25 @@ class zxAPITestCase(zxJDBCTestCase):
 		values = [(1996, 6, 22, 10, 11, 12), (2000, 11, 12, 3, 1, 12), (2001, 1, 12, 4, 9, 24)]
 		self._test_time(self.table("timestamptable"), zxJDBC.Timestamp, values, zxJDBC.TIMESTAMP, _cmp_)
 
+	def testOrderOfArgsMaxRowsOnly(self):
+		"""testing execute with max rows only"""
+		c = self.cursor()
+		try:
+
+			# maxrows only (SAPDB doesn't support maxrows as of version 7.2.0)
+			c.execute("select * from zxtesting", maxrows=3)
+			f = c.fetchall()
+			assert len(f) == 3, "expected length [3], got [%d]" % (len(f))
+
+		finally:
+			c.close()
+			self.db.commit()
+
 	def testOrderOfArgs(self):
 		"""testing execute with different argument orderings"""
 		c = self.cursor()
 		try:
 
-			# maxrows only
-			c.execute("select * from zxtesting", maxrows=3)
-			f = c.fetchall()
-			assert len(f) == 3, "expected length [3], got [%d]" % (len(f))
-
 			# bindings and params flipped
 			c.execute("select * from zxtesting where id = ?", bindings={0:zxJDBC.INTEGER}, params=[(3,)])
 			f = c.fetchall()
@@ -465,16 +762,17 @@ class zxAPITestCase(zxJDBCTestCase):
 		try:
 
 			c.execute("select * from zxtesting", maxrows=3)
-			f = c.fetchall()
-			assert len(f) == 3, "expected length [3], got [%d]" % (len(f))
+			self.assertEquals(3, len(c.fetchall()))
+
+			c.execute("select * from zxtesting where id > ?", (1,), maxrows=3)
+			self.assertEquals(3, len(c.fetchall()))
 
 			c.execute("select count(*) from zxtesting")
 			f = c.fetchall()
 			num = f[0][0]
 
 			c.execute("select * from zxtesting", maxrows=0)
-			f = c.fetchall()
-			assert len(f) == num, "expected length [%d], got [%d]" % (num, len(f))
+			self.assertEquals(num, len(c.fetchall()))
 
 		finally:
 			c.close()
@@ -509,19 +807,19 @@ class zxAPITestCase(zxJDBCTestCase):
 		finally:
 			c.close()
 
-	def testFetchingBeforeExecute(self):
-		"""testing fetch methods before execution"""
-		c = self.cursor()
-		try:
-			f = c.fetchall()
-			assert f == None, "expecting no results since no execute*() has been called"
-		finally:
-			c.close()
-
-	def testFetchingWithArraysize(self):
-		"""testing fetch methods using arraysize"""
-		c = self.cursor()
+	def _test_fetching(self, dynamic=0):
+		c = self.cursor(dynamic)
 		try:
+			# make sure None if the result is an empty result set
+			c.execute("select * from zxtesting where 1<0")
+			self.assertEquals(None, c.fetchone())
+			# make sure an empty sequence if the result is an empty result set
+			c.execute("select * from zxtesting where 1<0")
+			self.assertEquals([], c.fetchmany())
+			# make sure an empty sequence if the result is an empty result set
+			c.execute("select * from zxtesting where 1<0")
+			self.assertEquals([], c.fetchall())
+			# test some arraysize features
 			c.execute("select * from zxtesting")
 			f = c.fetchmany()
 			assert len(f) == c.arraysize, "expecting [%d] rows, got [%d]" % (c.arraysize, len(f))
@@ -536,6 +834,27 @@ class zxAPITestCase(zxJDBCTestCase):
 		finally:
 			c.close()
 
+	def testStaticFetching(self):
+		"""testing various static fetch methods"""
+		self._test_fetching(0)
+
+	def testDynamicFetching(self):
+		"""testing various dynamic fetch methods"""
+		self._test_fetching(1)
+
+	def testFetchingBeforeExecute(self):
+		"""testing fetch methods before execution"""
+		c = self.cursor()
+		try:
+			try:
+				c.fetchall()
+			except zxJDBC.Error, e:
+				pass
+			else:
+				self.fail("excepted exception calling fetchall() prior to execute()")
+		finally:
+			c.close()
+
 	def testBindingsWithNoParams(self):
 		"""testing bindings with no params"""
 		c = self.cursor()
@@ -558,7 +877,7 @@ class zxAPITestCase(zxJDBCTestCase):
 			c.close()
 
 	def testRowid(self):
-		"""test the autoincrement facilities of the different handlers"""
+		"""testing the autoincrement facilities of the different handlers"""
 		assert self.has_table("autoincrementtable"), "no autoincrement table"
 
 		c = self.cursor()
@@ -598,9 +917,48 @@ class zxAPITestCase(zxJDBCTestCase):
 			self.db.commit()
 			c.close()
 
-class LOBTest(zxJDBCTestCase):
+	def _test_fetchapi(self, dynamic=0):
+		"""Test the public Java API for Fetch"""
+		from com.ziclix.python.sql import Fetch, WarningListener
+		cur = self.cursor()
+		try:
+			c = self.db.__connection__
+			stmt = c.prepareStatement("select * from zxtesting where id < ?")
+			stmt.setInt(1, 5)
+			rs = stmt.executeQuery()
+			fetch = Fetch.newFetch(cur.datahandler, dynamic)
+			class WL(WarningListener):
+				def warning(self, event):
+					raise event.getWarning()
+			wl = WL()
+			fetch.addWarningListener(wl)
+			# the RS is closed by Fetch
+			fetch.add(rs)
 
-	def __blob(self, obj=0):
+			if not dynamic:
+				self.assertEquals(4, fetch.getRowCount())
+			assert fetch.fetchone()
+			assert fetch.fetchmany(2)
+			assert fetch.fetchall()
+			assert not fetch.fetchall()
+			self.assertEquals(4, fetch.getRowCount())
+			assert fetch.removeWarningListener(wl)
+			fetch.close()
+			stmt.close()
+		finally:
+			cur.close()
+
+	def testStaticFetchAPI(self):
+		"""Test static Java Fetch API"""
+		self._test_fetchapi(0)
+
+	def testDynamicFetchAPI(self):
+		"""Test dynamic Java Fetch API"""
+		self._test_fetchapi(1)
+
+class LOBTestCase(zxJDBCTestCase):
+
+	def _test_blob(self, obj=0):
 		assert self.has_table("blobtable"), "no blob table"
 		tabname, sql = self.table("blobtable")
 		fn = tempfile.mktemp()
@@ -653,13 +1011,13 @@ class LOBTest(zxJDBCTestCase):
 
 	def testBLOBAsString(self):
 		"""testing BLOB as string"""
-		self.__blob()
+		self._test_blob()
 
 	def testBLOBAsPyFile(self):
 		"""testing BLOB as PyFile"""
-		self.__blob(1)
+		self._test_blob(1)
 
-	def __clob(self, asfile=0):
+	def _test_clob(self, asfile=0):
 		assert self.has_table("clobtable"), "no clob table"
 		tabname, sql = self.table("clobtable")
 
@@ -695,11 +1053,11 @@ class LOBTest(zxJDBCTestCase):
 
 	def testCLOBAsString(self):
 		"""testing CLOB as string"""
-		self.__clob(0)
+		self._test_clob(0)
 
 	def testCLOBAsPyFile(self):
 		"""testing CLOB as PyFile"""
-		self.__clob(1)
+		self._test_clob(1)
 
 class BCPTestCase(zxJDBCTestCase):
 
@@ -727,7 +1085,7 @@ class BCPTestCase(zxJDBCTestCase):
 				self.db.rollback()
 				c.close()
 
-			dbSource = DBSource(src, self.datahandler, "zxtesting", None, None, None)
+			dbSource = DBSource(src, c.datahandler.__class__, "zxtesting", None, None, None)
 
 			cnt = Pipe().pipe(dbSource, csvSink) - 1 # ignore the header row
 
@@ -736,28 +1094,6 @@ class BCPTestCase(zxJDBCTestCase):
 			src.close()
 			os.remove(fn)
 
-	def _testXMLPipe(self):
-		"""testing the XML pipe"""
-		from java.io import PrintWriter, FileWriter
-		from com.ziclix.python.sql.pipe import Pipe
-		from com.ziclix.python.sql.pipe.db import DBSource
-		from com.ziclix.python.sql.pipe.xml import XMLSink
-
-		try:
-			src = self.connect()
-			fn = tempfile.mktemp(suffix="csv")
-			writer = PrintWriter(FileWriter(fn))
-			xmlSink = XMLSink(writer)
-
-			dbSource = DBSource(src, self.datahandler, "zxtesting", None, None, None)
-
-			cnt = Pipe().pipe(dbSource, xmlSink) - 1 # ignore the header row
-
-		finally:
-			writer.close()
-			src.close()
-			os.remove(fn)
-
 	def testDBPipe(self):
 		"""testing the DB pipe"""
 		from com.ziclix.python.sql.pipe import Pipe
@@ -775,8 +1111,8 @@ class BCPTestCase(zxJDBCTestCase):
 			one = c.fetchone()[0]
 			c.close()
 
-			dbSource = DBSource(src, self.datahandler, "zxtesting", None, None, None)
-			dbSink = DBSink(dst, self.datahandler, "zxtestingbcp", None, None, 1)
+			dbSource = DBSource(src, c.datahandler.__class__, "zxtesting", None, None, None)
+			dbSink = DBSink(dst, c.datahandler.__class__, "zxtestingbcp", None, None, 1)
 
 			cnt = Pipe().pipe(dbSource, dbSink) - 1 # ignore the header row
 
@@ -793,15 +1129,15 @@ class BCPTestCase(zxJDBCTestCase):
 			# this tests the internal assert in BCP.  we need to handle the case where we exclude
 			# all the rows queried (based on the fact no columns exist) but rows were fetched
 			# also make sure (eg, Oracle) that the column name case is ignored
-			dbSource = DBSource(src, self.datahandler, "zxtesting", None, ["id"], None)
-			dbSink = DBSink(dst, self.datahandler, "zxtestingbcp", ["id"], None, 1)
+			dbSource = DBSource(src, c.datahandler.__class__, "zxtesting", None, ["id"], None)
+			dbSink = DBSink(dst, c.datahandler.__class__, "zxtestingbcp", ["id"], None, 1)
 
 			self.assertRaises(zxJDBC.Error, Pipe().pipe, dbSource, dbSink)
 
 			params = [(4,)]
 
-			dbSource = DBSource(src, self.datahandler, "zxtesting", "id > ?", None, params)
-			dbSink = DBSink(dst, self.datahandler, "zxtestingbcp", None, None, 1)
+			dbSource = DBSource(src, c.datahandler.__class__, "zxtesting", "id > ?", None, params)
+			dbSink = DBSink(dst, c.datahandler.__class__, "zxtestingbcp", None, None, 1)
 
 			cnt = Pipe().pipe(dbSource, dbSink) - 1 # ignore the header row
 
